<?php
namespace Teknisa\FIS\Modules\Acessorias\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Fis07800DacAlFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    /*
     * Executa procedure para exportação de DAC-AL Fiscal [ARQUIVO_DAC].
     */
    public function callDacAlFiscalProcedure($params) {
        $procedure = new StoredProcedure($this->connection, 'ARQUIVO_DAC');

        $procedure->addParam(new Param('P_CDIMPOSTO', Param::PARAM_INPUT, $params["CDIMPOSTO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $params["CDEMPRESA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDINSCESTA', Param::PARAM_INPUT, $params["CDINSCESTA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $params["DTINICIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $params["DTFINAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VERSAO', Param::PARAM_INPUT, $params["VERSAO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RELEASE', Param::PARAM_INPUT, $params["RELEASE"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SBRELEASE', Param::PARAM_INPUT, $params["SBRELEASE"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TPDEC', Param::PARAM_INPUT, $params["TPDEC"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PDEC', Param::PARAM_INPUT, $params["PDEC"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INDMOV', Param::PARAM_INPUT, $params["INDMOV"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INDENC', Param::PARAM_INPUT, $params["INDENC"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_GUIA', Param::PARAM_INPUT, $params["GUIA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_APURAC', Param::PARAM_INPUT, $params["APURAC"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SISTEMA', Param::PARAM_INPUT, $params["SISTEMA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CALC', Param::PARAM_INPUT, $params["CALC"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_GRUPPROD', Param::PARAM_INPUT, $params["GRUPPROD"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_AIDF', Param::PARAM_INPUT, $params["AIDF"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_REGANUAL', Param::PARAM_INPUT, $params["REGANUAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQARQMAG', Param::PARAM_INPUT, $params["NRSEQARQMAG"], Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }

    /*
     * Função para excluir os dados gerados na procedure de DAC-AL Fiscal.
     */
    public function deleteDacAlFiscalData($params) {
        // Query a ser executada.
        $sql = "DELETE FROM ARQMAGNETICO WHERE NRSEQARQMAG = :NRSEQARQMAG AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para retornar a sigla do estado relacionado a uma inscrição estadual.
     */
    public function getEstadoInscricaoEstadual($params) {
        // Query a ser executada.
        $sql = "SELECT SGESTADO FROM INSCRESTAD WHERE CDEMPRESA = :CDEMPRESA AND CDINSCESTA = :CDINSCESTA AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch no primeiro dado, retornando-o.
        return $statement->fetch(\PDO::FETCH_ASSOC);
    }

    /*
     * Função para retornar os dados gerados na procedure de DAC-AL Fiscal.
     */
    public function getDacAlFiscalData($params) {
        // Query a ser executada.
        $sql = "SELECT NRSEQARQMAG, NRORDREGARQ, DSREGARQMAG as DSREGARQMAG 
                FROM ARQMAGNETICO 
                WHERE NRSEQARQMAG = :NRSEQARQMAG 
                    AND NRORG = :NRORG 
                ORDER BY NRORDREGARQ, SUBSTR(DSREGARQMAG,1,17)";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch no primeiro dado, retornando-o.
        return $statement->fetchAll();
    }
    
    public function findAgendamento() {
        // Query a ser executada.
        $sql = "SELECT 
                    A.NRORG,
                    A.NRAGENDAMENTO,
                    A.CDTELA,
                    A.CDEMPRESA,
                    E.NMEMPRESA,
                    A.CDINSCESTA,
                    A.DTAGENDAMENTO,
                    A.DTINICIAL,
                    A.DTFINAL,
                    DECODE(A.IDPERIAGEN, '0', A.DTINICIAL,
                                           '1', TO_CHAR(SYSDATE, 'DD/MM/YYYY'),
                                           '2', TO_CHAR(ADD_MONTHS(SYSDATE,(-1)), 'DD/MM/YYYY')) AS DTREFINI,
                    A.IDPERIAGEN,
                    A.IDPERIAGEN,
                    A.IDSTATUS
                    FROM AGENDOBRIGACESS A,
                    EMPRESA E
                    WHERE CDTELA  = 'FIS07800'
                    AND A.CDEMPRESA = E.CDEMPRESA ";
                    

        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute();
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function insertAgendamento($row) {
        $sql = "INSERT INTO AGENDOBRIGACESS
            ( 
                NRORG,
                NRAGENDAMENTO,
                CDTELA,
            	CDEMPRESA,
                CDINSCESTA,
            	DTAGENDAMENTO,
            	DTINICIAL,
            	DTFINAL,
            	IDPERIAGEN,
            	
                DTCAMPO1   ) 
             VALUES (
                :NRORG,
                :NRAGENDAMENTO,
                :CDTELA,
                :CDEMPRESA,
                :CDINSCESTA,
                :DTAGENDAMENTO,
                :DTINICIAL,
                :DTFINAL,
                :IDPERIAGEN,
                
                :DTCAMPO1 )";
                
        $params = array(
            "NRORG"         => $row["NRORG"],
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "DTAGENDAMENTO" => $row["DTAGENDAMENTO"],
            
            //Campos da tabela que serão salvos e os campos da declaração
            "DTINICIAL" => $row["DTINICIAL"],
            "DTFINAL"   => $row["DTFINAL"],
            "IDPERIAGEN"=> $row["IDPERIAGEN"],
            
            "DTCAMPO1"  => $row["DTREFINI"]
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateAgendamento($row) {
        $sql = "UPDATE AGENDOBRIGACESS
                    SET CDEMPRESA = :CDEMPRESA,
                        CDINSCESTA = :CDINSCESTA,
                    	DTAGENDAMENTO = :DTAGENDAMENTO,
                    	DTINICIAL = :DTINICIAL,
                    	DTFINAL = :DTFINAL,
                    	IDPERIAGEN = :IDPERIAGEN,
                        DTCAMPO1 = :DTCAMPO1
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO
                    AND CDTELA = :CDTELA";
                    
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "DTAGENDAMENTO" => $row["DTAGENDAMENTO"],
            
            //Campos da tabela que serão salvos e os campos da declaração
            "DTINICIAL"     => $row["DTINICIAL"],
            "DTFINAL"       => $row["DTFINAL"],
            "IDPERIAGEN"    => $row["IDPERIAGEN"],
            
            "DTCAMPO1"      => $row["DTREFINI"]
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateAgendamentoIdStatus($row, $idstatus) {
        $sql = "UPDATE AGENDOBRIGACESS
                    SET IDSTATUS = :IDSTATUS
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO
                    AND CDTELA = :CDTELA";
                    
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "IDSTATUS"      => $idstatus
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getInscEsta($row) {
        // Query a ser executada.
        $sql = "SELECT CDINSCESTA FROM INSCRESTAD WHERE CDEMPRESA = :CDEMPRESA";

        $params = array(
            'CDEMPRESA' => $row['CDEMPRESA']
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function getAgendamentoInsert($row) {
        // Query a ser executada.
        $sql = "SELECT * FROM AGENDOBRIGACESS 
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO";

        $params = array(
            "NRAGENDAMENTO"=>$row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function getConfiguraSMTP($row)  {
        // Query a ser executada.
        $sql = "SELECT * FROM CONFIGURASMTP
                    WHERE NRORG = :NRORG";
        
        $params = array(
            "NRORG"=> $row["NRORG"]
        );
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetchAll();
    }
    
    public function getContatos($row)  {
        // Query a ser executada.
        $sql = "SELECT * FROM CONTATOAPURAC
                    WHERE NRORG = :NRORG
                    AND CDEMPRESA = :CDEMPRESA
                    AND IDEXPORTAUTO = 'S'";
        
        $params = array(
            "NRORG"=> $row["NRORG"],
            "CDEMPRESA"=> $row["CDEMPRESA"]
        );
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetchAll();
    }
    
    public function getAgendamentoArquivo($row) {
        $sql = "SELECT * FROM ARQUIOBRIGACESS
                WHERE NRAGENDAMENTO = :NRAGENDAMENTO";
        
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function deleteAgendamentoArquivo($row) {
        $sql = "DELETE FROM ARQUIOBRIGACESS
                WHERE NRAGENDAMENTO = :NRAGENDAMENTO";
        
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    }
  
    public function deleteAgendamento($row) {
        // Query a ser executada.
        $sql = "DELETE FROM AGENDOBRIGACESS 
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO";

        $params = array(
            "NRAGENDAMENTO"=>$row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    }
    
    public function findAgendamentoExec($row) {
        // Query a ser executada.
        $sql = "SELECT 
                    TO_CHAR(SYSDATE, 'DD/MM/YYYY HH24:MI') AS DTATUAL,
                    TO_CHAR(ADD_MONTHS(SYSDATE,(-1)), 'DD/MM/YYYY HH24:MI') AS LAST_MONTH,
                    A.NRAGENDAMENTO,
                    A.CDTELA,
                    A.CDEMPRESA,
                    A.CDINSCESTA,
                    A.DTAGENDAMENTO,
                    A.DTINICIAL,
                    A.DTFINAL,
                    A.DTCAMPO1 AS DTREFINI,
                    A.IDPERIAGEN,
                    A.IDSTATUS
                    FROM AGENDOBRIGACESS A
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO";
        
        $params = array(
            "NRAGENDAMENTO"=> $row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetchAll();
    }
    
    public function findParamDeclAcess($row) {
        // Query a ser executada.
        $sql = "SELECT
                 CDCAMPO1       AS CDGRUPPROD,
                 CDCAMPO2       AS VERSAO,
                 CDCAMPO3       AS RELEASE,
                 CDCAMPO4       AS SBRELEASE,
                 CDCAMPO5       AS TPDEC,
                 CDCAMPO6       AS PDEC,
                 CDCAMPO7       AS INDMOV,
                 CDCAMPO8       AS INDENC,
                 CDCAMPO9       AS GUIA,
                 CDCAMPO10      AS APURAC,
                 CDCAMPO11      AS SISTEMA,
                 CDCAMPO12      AS CALC,
                 CDCAMPO13      AS REGANUAL
             FROM
                 PARAMDECLACES
             WHERE
                 CDTELA =:CDTELA
                 AND CDEMPRESA =:CDEMPRESA";
        
        $params = array(
            "CDTELA"=> $row["CDTELA"],
            "CDEMPRESA"=> $row["CDEMPRESA"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetch();
    }
    
    public function getNmEmpresa($row) {
        // Query a ser executada.
        $sql = "SELECT NMEMPRESA FROM EMPRESA
                    WHERE CDEMPRESA = :CDEMPRESA
                    AND NRORG = :NRORG";
        
        $params = array(
            "CDEMPRESA"=> $row["CDEMPRESA"],
            "NRORG"=> $row["NRORG"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetch();
    }
    
    public function verAgendamentoArquivo($row) {
        $sql = "SELECT * FROM ARQUIOBRIGACESS
                WHERE NRAGENDAMENTO = :NRAGENDAMENTO
                AND NMARQUIVO = :NMARQUIVO";
        
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "NMARQUIVO" => $row["NMARQUIVO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function insertAgendamentoArquivo($row) {
        $sql = "INSERT INTO ARQUIOBRIGACESS
            ( 
                NRORG,
                NRARQUIVO,
                NRAGENDAMENTO,
                CDTELA,
                CDEMPRESA,
                CDINSCESTA,
                DTAGENDAMENTO,
                DTEXECAGEND,
                NMARQUIVO,
                NMPATHARQ
            )
             VALUES (
                :NRORG,
                :NRARQUIVO,
                :NRAGENDAMENTO,
                :CDTELA,
                :CDEMPRESA,
                :CDINSCESTA,
                :DTAGENDAMENTO,
                :DTEXECAGEND,
                :NMARQUIVO,
                :NMPATHARQ  )";
        
        $params = array(
            "NRORG"         => $row["NRORG"],
            "NRARQUIVO"     => $row["NRARQUIVO"],
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "DTAGENDAMENTO" => $row["DTAGENDAMENTO"],
            "DTEXECAGEND"   => $row["DTEXECAGEND"],
            "NMARQUIVO"     => $row["NMARQUIVO"],
            "NMPATHARQ"     => $row["NMPATHARQ"]
            
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

}