<?php

namespace Teknisa\FIS\Modules\Apuracoes\Controllers;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Modules\Apuracoes\Services\Fis00100ApuracaoDeImpostoService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;

class Fis00100ApuracaoDeImpostoController extends \Teknisa\FIS\Controllers\TeknisaCrud{
    protected $crudService;
    protected $dataSourceName;
    protected $dataSourceNameToSave  = false;
    /** @var ParameterBag */
    protected $parameterBag;
    /** @var string[] */
    protected $parameterBagColumns = array();

    public function __construct($crudService, ParameterBag $parameterBag, EntityManager $entityManager, Fis00100ApuracaoDeImpostoService $serviceApuracaoImposto) {
        $this->entityManager            = $entityManager;
        $this->serviceApuracaoImposto = $serviceApuracaoImposto;
        parent::__construct($crudService, $parameterBag);
    }

    
    public function apuraMapasContribuicoes(DTO\Request\Row $request, DTO\Response $response) {
        try {
            set_time_limit(14400);
            $row = $request->getRow();
            $avisos = $this->serviceApuracaoImposto->apuraMapasContribuicoes($row);
            // Retorna o array de avisos da apuração.
            $response->addDataSet(new DataSet("avisos", $avisos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function apurarMapa (DTO\Request\Row $request, DTO\Response $response) {
        try {
            set_time_limit(14400);
            $row = $request->getRow();
            $avisos = $this->serviceApuracaoImposto->apurarMapa($row);
            // Retorna o array de avisos da apuração.
            $response->addDataSet(new DataSet("avisos", $avisos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function bloquearMapa (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->bloquearMapa($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function buscaPrxDataIni (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $prxDataIni = $this->serviceApuracaoImposto->buscaPrxDataIniECodigoMapa($row);
            $response->addDataSet(new DataSet("result", $prxDataIni));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function checkUltimoMapa(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $isUltimoMapa = $this->serviceApuracaoImposto->checkUltimoMapa($row);
            $response->addDataSet(new DataSet("validation", array("isUltimoMapa" => $isUltimoMapa)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function checkValidSequencialDetalhamento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $isValid = $this->serviceApuracaoImposto->checkValidSequencialDetalhamento($row);
            $response->addDataSet(new DataSet("validation", array("isValid" => $isValid)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteAjustesDetalhamento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);

            $this->dataSourceName = 'ajusteapurcontr_detalhamento_contribuicoes';
            parent::delete($request, $response);

            $this->serviceApuracaoImposto->calculaDetalhamentoContribuicoes($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deleteItensDetalhamento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);

            $this->dataSourceName = 'itdetapucontr_detalhamento_contribuicoes';
            parent::delete($request, $response);

            $this->serviceApuracaoImposto->calculaDetalhamentoContribuicoes($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deleteMapa (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);
            $this->serviceApuracaoImposto->deleteMapa($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function desbloquearMapa (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->desbloquearMapa($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findAjustesDetalhamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'ajusteapurcontr_detalhamento_contribuicoes';
            $this->parameterBagColumns = array("IDTPAJUSTE", "IDINDICAJUSTE", "NRSEQUMAPAPU" ,"NRORG");

            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function findItensDetalhamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'itdetapucontr_detalhamento_contribuicoes';
            $this->parameterBagColumns = array("IDTIPODETAPU", "NRSEQUMAPAPU" ,"NRORG");

            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function findCompensacoesDetalhamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'itdetapucontr_detalhamentoCompensacoes';
            $this->parameterBagColumns = array("IDTIPODETAPU", "NRSEQUMAPAPU" ,"NRORG");

            parent::find($request, $response);
            $dataSets = $response->getDataSets();

            if(!empty($dataSets)) {
                $rows = $dataSets[0]->getRows();
                $dados = $this->serviceApuracaoImposto->aplicarMascaraInscricao($rows);
                $dataSet = new DataSet("itdetapucontr_detalhamentoCompensacoes", $dados);
                $response->setDataSets(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function findNotasFiscaisMapaGeral(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'notasfiscais_apuracaoimposto_geral';
            $this->parameterBagColumns = array("NRSEQUMAPAPU");

            parent::find($request, $response);
            $dataSets = $response->getDataSets();

            if(!empty($dataSets)) {
                $rows = $dataSets[0]->getRows();
                $dados = $this->serviceApuracaoImposto->aplicarMascaraInscricao($rows);
                $dataSet = new DataSet("notasfiscais_apuracaoimposto_geral", $dados);
                $response->setDataSets(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function findNotasFiscaisGeral(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'notasfiscais_geral';
            $this->parameterBagColumns = array("NRSEQUMAPAPU");

            parent::find($request, $response);
            $dataSets = $response->getDataSets();

            if(!empty($dataSets)) {
                $rows = $dataSets[0]->getRows();
                $dados = $this->serviceApuracaoImposto->aplicarMascaraInscricao($rows);
                $dataSet = new DataSet("notasfiscais_geral", $dados);
                $response->setDataSets(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function findNotasFiscaisMapaSimples(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'notasfiscais_apuracaoimposto_simples';
            $this->parameterBagColumns = array("NRSEQUMAPAPU");

            parent::find($request, $response);
            $dataSets = $response->getDataSets();

            if(!empty($dataSets)) {
                $rows = $dataSets[0]->getRows();
                $dados = $this->serviceApuracaoImposto->aplicarMascaraInscricao($rows);
                $dataSet = new DataSet("notasfiscais_apuracaoimposto_simples", $dados);
                $response->setDataSets(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getNextSequencialAjusteContribuicoes(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();

            $codigo = $this->serviceApuracaoImposto->getNextSequencialAjusteContribuicoes($row);
            $response->addDataSet(new DataSet("codigo", array("NRSEQAJUSTE" => $codigo)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveAjustesDetalhamento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);

            $this->dataSourceName = 'ajusteapurcontr';
            parent::save($request, $response);

            $this->serviceApuracaoImposto->calculaDetalhamentoContribuicoes($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveAjusteContribuicoes(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);

            $this->serviceApuracaoImposto->updateAjusteApurContr($row);
            //$this->dataSourceName = 'ajusteapurcontr_detalhamento_contribuicoes';
            //parent::save($request, $response);

            $this->serviceApuracaoImposto->calculaDetalhamentoContribuicoes($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveArrecadacaoContribuicoes(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);
            $this->serviceApuracaoImposto->saveArrecadacaoContribuicoes($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarArrecadacaoGeral(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);
            $this->serviceApuracaoImposto->salvarArrecadacaoGeral($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveItensDetalhamento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);

            $this->dataSourceName = 'itdetapucontr_detalhamento_contribuicoes';
            parent::save($request, $response);

            $this->serviceApuracaoImposto->calculaDetalhamentoContribuicoes($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveMapa(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            // Impede o procedimento se o mapa estiver bloqueado.
            $this->serviceApuracaoImposto->validaMapaBloqueado($row);

            // Seta o dataSourceName a ser utilizado conforme tipo de apuração
            // do mapa.
            $this->dataSourceName = $this->serviceApuracaoImposto->isSimplesNacional($row) ? 'mapaapuracsn' : 'mapaapurac';
            parent::save($request, $response);

            // Verifica se é um mapa novo sendo inserido.
            if ($row["__is_new"]) {
                // Insere a tabela de detalhamento para o mapa.
                $this->serviceApuracaoImposto->insertDetalhamento($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function validaImpostosRelacionados (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $validation = $this->serviceApuracaoImposto->validaImpostosRelacionados($row['CDIMPOSTONFE']);
            $response->addDataSet(new DataSet("validation", $validation));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvarDetalhamento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->updateDetalhamento($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarDetalhamentoInss(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $this->crudService->getRowsToSave($request, 'mapaapurac');
            $this->serviceApuracaoImposto->salvarDetalhamentoInss($rows[0]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    
    
    public function salvarCreditoAnteriorDifalFcp(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->salvarCreditoAnteriorDifalFcp($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    

    public function salvarGuiasComplementares(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->salvarGuiasComplementares($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    
    public function salvarSubDetalhamentoInss(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $this->crudService->getRowsToSave($request, 'itdetapucontr');
            $this->serviceApuracaoImposto->salvarSubDetalhamentoInss($rows[0]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    public function salvarSubDetalhamentoImpostosGerais(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $this->crudService->getRowsToSave($request, 'itmaparecup');
            $rows[0]['IDCOMPOEC197'] = $rows[0]['IDCOMPOEC197'] == "" ? "N" : $rows[0]['IDCOMPOEC197'];
            $this->serviceApuracaoImposto->salvarSubDetalhamentoImpostosGerais($rows[0]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarSubDetalhamentoIcmsDifalFcp(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $this->crudService->getRowsToSave($request, 'itmaparecupnc');
            $this->serviceApuracaoImposto->salvarSubDetalhamentoIcmsDifalFcp($rows[0]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
  
    public function deletarSubDetalhamentoImpostosGerais(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->serviceApuracaoImposto->deletarSubDetalhamentoImpostosGerais($request->getRow());
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deletarSubDetalhamentoDifalFcp(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->serviceApuracaoImposto->deletarSubDetalhamentoDifalFcp($request->getRow());
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarItensComposicoes(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $soma = $this->serviceApuracaoImposto->salvarItensComposicoes($row);
            $response->addDataSet(new DataSet("soma", $soma));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function controlaValorItensComposicao(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $soma = $this->serviceApuracaoImposto->controlaValorItensComposicao($row);
            $response->addDataSet(new DataSet("soma", $soma));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function salvarInformacoesAdicionais(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->salvarInformacoesAdicionais($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function replicarInformacoesAdicionais(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->replicarInformacoesAdicionais($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }    
    
    public function salvarArrecadacaoIcmsSt(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->salvarArrecadacaoIcmsSt($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }    
    
    public function salvarSubDetalhamentoIcmsSt(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $this->crudService->getRowsToSave($request, 'ajustmapaapursg');
            $this->serviceApuracaoImposto->salvarSubDetalhamentoIcmsSt($rows[0]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function geraGiaRs(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            $this->serviceApuracaoImposto->geraGiaRs($row);
            
            $this->entityManager->commit();

        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }    
    
    public function validaImpostoApurac(DTO\Request $request, DTO\Response $response){
         try {
            $row = $request->getRow();
            $this->serviceApuracaoImposto->validaImpostoApurac($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    
}